# Copyright 2026 Arturo Roberti
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import TypeAlias, TypedDict, Union


class ResolvedArgsDefinition(TypedDict):
    """Dictionary representing argument definitions for tasks."""

    # fmt: off
    help:        str
    default:     None | bool | str | list[str]
    choices:     None | list[str]
    nargs:       None | Union[int, str]
    mutex:       None | str
    # fmt: on


ResolvedArgsDefinitionCollection: TypeAlias = dict[str, ResolvedArgsDefinition]
