# Copyright 2026 Arturo Roberti
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import NotRequired, TypeAlias, TypedDict

from ..remotes import GitQuery
from ..tasks import (
    CustomTaskDictCollection,
    DefaultTaskDictCollection,
    ResolvedCustomTaskDictCollection,
    ResolvedDefaultTaskDictCollection,
)

# NOTE: The key "default" is required
PluginOptions: TypeAlias = dict[str, CustomTaskDictCollection]
ResolvedPluginOptions: TypeAlias = dict[str, ResolvedCustomTaskDictCollection]


class PluginManifest(TypedDict):
    # fmt: off
    imports: NotRequired[list[GitQuery | str]]
    tasks:   NotRequired[DefaultTaskDictCollection]
    options: PluginOptions
    # fmt: on


class ResolvedPluginManifest(TypedDict):
    # fmt: off
    imports: list[GitQuery | str]
    tasks:   ResolvedDefaultTaskDictCollection
    options: ResolvedPluginOptions
    # fmt: on
